package com.qmth.boot.api.config;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.core.enums.Platform;
import com.qmth.boot.core.rateLimit.entity.RateLimitRule;
import com.qmth.boot.tools.signature.SignatureType;
import org.apache.commons.lang3.ArrayUtils;

import java.util.ArrayList;
import java.util.List;

public class AacConfig {

    public static final String[] EMPTY_STRING_ARRAY = new String[] {};

    public static final Platform[] EMPTY_PLATFORM_ARRAY = new Platform[] {};

    public static final RateLimitRule[] EMPTY_RATE_LIMIT_CONFIG = new RateLimitRule[] {};

    public static final SignatureType[] EMPTY_SIGNATURE_TYPE = new SignatureType[] {};

    private Boolean strict;

    private Platform[] platform = EMPTY_PLATFORM_ARRAY;

    private String[] ipAllow = EMPTY_STRING_ARRAY;

    private String[] ipDeny = EMPTY_STRING_ARRAY;

    private RateLimitRule[] rateLimit = EMPTY_RATE_LIMIT_CONFIG;

    private Boolean auth;

    private SignatureType[] signType = EMPTY_SIGNATURE_TYPE;

    public AacConfig(ApiProperties properties) {
        this.strict = properties.isGlobalStrict();
        this.auth = properties.isGlobalAuth();

        buildRateLimit(properties.getGlobalRateLimit());
    }

    public AacConfig(Aac annotation) {
        if (annotation == null) {
            return;
        }
        if (annotation.strict() != BOOL.NULL) {
            this.strict = (annotation.strict() == BOOL.TRUE);
        }
        if (annotation.platform().length > 0) {
            this.platform = annotation.platform();
        }
        if (annotation.auth() != BOOL.NULL) {
            this.auth = (annotation.auth() == BOOL.TRUE);
        }
        if (annotation.ipAllow().length > 0) {
            this.ipAllow = annotation.ipAllow();
        }
        if (annotation.ipDeny().length > 0) {
            this.ipDeny = annotation.ipDeny();
        }
        if (annotation.signType().length > 0) {
            this.signType = annotation.signType();
        }

        buildRateLimit(annotation.rateLimit());
    }

    public void merge(AacConfig config) {
        if (config == null) {
            return;
        }
        if (config.strict != null) {
            this.strict = config.strict;
        }
        if (ArrayUtils.isNotEmpty(config.platform)) {
            this.platform = config.platform;
        }
        if (config.auth != null) {
            this.auth = config.auth;
        }
        if (ArrayUtils.isNotEmpty(config.ipAllow)) {
            this.ipAllow = config.ipAllow;
        }
        if (ArrayUtils.isNotEmpty(config.ipDeny)) {
            this.ipDeny = config.ipDeny;
        }
        if (ArrayUtils.isNotEmpty(config.signType)) {
            this.signType = config.signType;
        }
        if (ArrayUtils.isNotEmpty(config.rateLimit)) {
            this.rateLimit = config.rateLimit;
        }
    }

    public boolean isStrict() {
        return strict;
    }

    public Platform[] getPlatform() {
        return platform;
    }

    public String[] getIpAllow() {
        return ipAllow;
    }

    public String[] getIpDeny() {
        return ipDeny;
    }

    public RateLimitRule[] getRateLimit() {
        return rateLimit;
    }

    public boolean isAuth() {
        return auth;
    }

    public SignatureType[] getSignType() {
        return signType;
    }

    private void buildRateLimit(String[] expressions) {
        if (ArrayUtils.isNotEmpty(expressions)) {
            List<RateLimitRule> list = new ArrayList<>(expressions.length);
            for (String expression : expressions) {
                RateLimitRule config = RateLimitRule.parse(expression);
                if (config != null) {
                    list.add(config);
                }
            }
            if (!list.isEmpty()) {
                this.rateLimit = list.toArray(new RateLimitRule[0]);
            }
        }
    }

}
